searchData={"items":[{"type":"module","title":"Uploadex","doc":"Imports functions from Files, Context and Resolver modules, passing itself as a parameter for configuration.","ref":"Uploadex.html"},{"type":"module","title":"Uploadex.Context","doc":"Context Helper functions for handling files.\n\nNote that all functions in this module require the Uploader as an argument. You are free to call them like that:\n\n    iex> Uploadex.Context.create_with_file(changeset, Repo, MyUploader)\n    {:ok, %User{}}\n\nHowever, by doing `use Uploadex, repo: Repo` in your uploader, you can call these functions directly through the uploader to avoid having to pass these\nextra arguments around:\n\n    iex> MyUploader.create_with_file(changeset)\n    {:ok, %User{}}","ref":"Uploadex.Context.html"},{"type":"function","title":"Uploadex.Context.create_with_file/4","doc":"Inserts the changeset and store the record files in a database transaction,\nso if the files fail to be stored the record will not be created.","ref":"Uploadex.Context.html#create_with_file/4"},{"type":"function","title":"Uploadex.Context.delete_with_file/4","doc":"Deletes the record and all of its files.\nThis is not in a database transaction, since the delete operation never returns errors.","ref":"Uploadex.Context.html#delete_with_file/4"},{"type":"function","title":"Uploadex.Context.update_with_file/5","doc":"Updates the record and its files in a database transaction,\nso if the files fail to be stored the record will not be created.\n\nThis function also deletes files that are no longer referenced.","ref":"Uploadex.Context.html#update_with_file/5"},{"type":"function","title":"Uploadex.Context.update_with_file_keep_previous/4","doc":"Similar to `update_with_file/3`, but does not delete previous files.","ref":"Uploadex.Context.html#update_with_file_keep_previous/4"},{"type":"module","title":"Uploadex.FileProcessing","doc":"Process files in Base64","ref":"Uploadex.FileProcessing.html"},{"type":"function","title":"Uploadex.FileProcessing.process_binary/1","doc":"If it's in base64, decode it. Otherwise, do not try to process the file.","ref":"Uploadex.FileProcessing.html#process_binary/1"},{"type":"type","title":"Uploadex.FileProcessing.processed_binary/0","doc":"","ref":"Uploadex.FileProcessing.html#t:processed_binary/0"},{"type":"module","title":"Uploadex.FileStorage","doc":"Storage for Local Files.","ref":"Uploadex.FileStorage.html"},{"type":"module","title":"Opts - Uploadex.FileStorage","doc":"* `directory`: String (required for all functions) - Relative to `base_path`\n* `base_path`: String (required for all functions)\n* `base_url`:  String (required for `c:Uploadex.Storage.get_url/2`)\n\nTo build the URL, `base_path` will be replaced by `base_url`.","ref":"Uploadex.FileStorage.html#module-opts"},{"type":"module","title":"Example - Uploadex.FileStorage","doc":"To use this storage for your `User` record, define these functions in your `Uploadex.Uploader` implementation:\n\n    def default_opts(Uploadex.FileStorage), do: [base_path: :code.priv_dir(:my_app), base_url: Endpoint.url()]\n\n    def storage(%User{} = user, :photo), do: {Uploadex.FileStorage, directory: \"/uploads/users\"}","ref":"Uploadex.FileStorage.html#module-example"},{"type":"module","title":"Uploadex.Files","doc":"Functions to store and delete files.\n\nNote that all functions in this module require the Uploader as an argument. You are free to call them like that:\n\n    iex> Uploadex.Files.store_files(user, MyUploader)\n    {:ok, %User{}}\n\nHowever, by doing `use Uploadex` in your uploader, you can call these functions directly through the uploader to avoid having to pass this\nextra argument around:\n\n    iex> MyUploader.store_files(user)\n    {:ok, %User{}}","ref":"Uploadex.Files.html"},{"type":"function","title":"Uploadex.Files.delete_files/2","doc":"Deletes all files for a record.","ref":"Uploadex.Files.html#delete_files/2"},{"type":"function","title":"Uploadex.Files.delete_previous_files/3","doc":"Deletes all files that changed.","ref":"Uploadex.Files.html#delete_previous_files/3"},{"type":"function","title":"Uploadex.Files.get_file_url/4","doc":"","ref":"Uploadex.Files.html#get_file_url/4"},{"type":"function","title":"Uploadex.Files.get_files_url/3","doc":"","ref":"Uploadex.Files.html#get_files_url/3"},{"type":"function","title":"Uploadex.Files.get_files_url/4","doc":"","ref":"Uploadex.Files.html#get_files_url/4"},{"type":"function","title":"Uploadex.Files.get_temporary_file/5","doc":"","ref":"Uploadex.Files.html#get_temporary_file/5"},{"type":"function","title":"Uploadex.Files.get_temporary_files/4","doc":"","ref":"Uploadex.Files.html#get_temporary_files/4"},{"type":"function","title":"Uploadex.Files.get_temporary_files/5","doc":"","ref":"Uploadex.Files.html#get_temporary_files/5"},{"type":"function","title":"Uploadex.Files.store_files/2","doc":"Stores all files of a record, as defined by the uploader.\n\nFiles that are not maps are ignored, which allows for assigning an existing file to a record without recreating it, by simply passing it's filename.","ref":"Uploadex.Files.html#store_files/2"},{"type":"function","title":"Uploadex.Files.store_files/3","doc":"","ref":"Uploadex.Files.html#store_files/3"},{"type":"type","title":"Uploadex.Files.record/0","doc":"","ref":"Uploadex.Files.html#t:record/0"},{"type":"type","title":"Uploadex.Files.record_field/0","doc":"","ref":"Uploadex.Files.html#t:record_field/0"},{"type":"type","title":"Uploadex.Files.status/0","doc":"","ref":"Uploadex.Files.html#t:status/0"},{"type":"module","title":"Uploadex.Resolver","doc":"Resolver functions to make it easier to use Uploadex with Absinthe.\n\nNote that all functions in this module require the Uploader as an argument. You are free to call them like that:","ref":"Uploadex.Resolver.html"},{"type":"module","title":"Example - Uploadex.Resolver","doc":"In your Absinthe schema, assuming user only has one photo:\n\n    object :user do\n      field :photo_url, :string, resolve: Uploadex.Resolver.get_file_url(:photo, MyUploader)\n    end\n\n  If it has many photos:\n\n    object :user do\n      field :photos, list_of(:string), resolve: Uploadex.Resolver.get_files_url(:photos, MyUploader)\n    end\n\nHowever, by doing `use Uploadex` in your uploader, you can call these functions directly through the uploader to avoid having to pass this\nextra argument around:","ref":"Uploadex.Resolver.html#module-example"},{"type":"module","title":"Example - Uploadex.Resolver","doc":"In your Absinthe schema, assuming user only has one photo:\n\n    object :user do\n      field :photo_url, :string, resolve: MyUploader.get_file_url(:photo)\n    end\n\n  If it has many photos:\n\n    object :user do\n      field :photos, list_of(:string), resolve: MyUploader.get_files_url(:photos)\n    end","ref":"Uploadex.Resolver.html#module-example"},{"type":"function","title":"Uploadex.Resolver.get_file_url/2","doc":"","ref":"Uploadex.Resolver.html#get_file_url/2"},{"type":"function","title":"Uploadex.Resolver.get_files_url/2","doc":"","ref":"Uploadex.Resolver.html#get_files_url/2"},{"type":"module","title":"Uploadex.S3Storage","doc":"Storage for AWS S3.","ref":"Uploadex.S3Storage.html"},{"type":"module","title":"Opts - Uploadex.S3Storage","doc":"* `bucket`: String (required for all functions)\n* `region`:  String (required for `c:Uploadex.Storage.get_url/2`)\n* `directory`: String (required for all functions)\n* `upload_opts`: Keyword list. This opts are passed to `ExAws.S3.upload/4` and `ExAws.S3.put_object/4` (required for `c:Uploadex.Storage.store/2`).\n    If `content_type` is not specified in `upload_opts`, the default is the upload's content type.","ref":"Uploadex.S3Storage.html#module-opts"},{"type":"module","title":"Example - Uploadex.S3Storage","doc":"To use this storage for your `User` record, define these functions in your `Uploadex.Uploader` implementation:\n\n    def default_opts(Uploadex.S3Storage), do: [bucket: \"my_bucket\", region: \"sa-east-1\", upload_opts: [acl: :public_read]]\n\n    def storage(%User{} = user, :photo), do: {Uploadex.S3Storage, directory: \"/photos\"}","ref":"Uploadex.S3Storage.html#module-example"},{"type":"behaviour","title":"Uploadex.Storage","doc":"Behaviour for a Storage implementation.","ref":"Uploadex.Storage.html"},{"type":"callback","title":"Uploadex.Storage.delete/2","doc":"Deletes the file","ref":"Uploadex.Storage.html#c:delete/2"},{"type":"callback","title":"Uploadex.Storage.get_temporary_file/3","doc":"Returns the path of a temporary file, that will be deleted after the configured amount of time.","ref":"Uploadex.Storage.html#c:get_temporary_file/3"},{"type":"callback","title":"Uploadex.Storage.get_url/2","doc":"Returns the file's URL","ref":"Uploadex.Storage.html#c:get_url/2"},{"type":"callback","title":"Uploadex.Storage.store/2","doc":"Stores the file","ref":"Uploadex.Storage.html#c:store/2"},{"type":"type","title":"Uploadex.Storage.file/0","doc":"","ref":"Uploadex.Storage.html#t:file/0"},{"type":"type","title":"Uploadex.Storage.opts/0","doc":"","ref":"Uploadex.Storage.html#t:opts/0"},{"type":"type","title":"Uploadex.Storage.record/0","doc":"","ref":"Uploadex.Storage.html#t:record/0"},{"type":"type","title":"Uploadex.Storage.status/0","doc":"","ref":"Uploadex.Storage.html#t:status/0"},{"type":"module","title":"Uploadex.TestStorage","doc":"Storage to be used for testing. It holds the files in memory (using a `Agent`).","ref":"Uploadex.TestStorage.html"},{"type":"module","title":"Configuring - Uploadex.TestStorage","doc":"To configure your app to use the `Uploadex.TestStorage`, just add this storage to your `Uploadex` module,\ndepending on the current environment:\n\n    defmodule MyApp.Uploader do\n      use Uploadex\n\n      @impl true\n      def get_fields(%User{}), do: :files\n\n      @impl true\n      def default_opts(Uploadex.FileStorage), do: [...]\n      def default_opts(Uploadex.TestStorage), do: []\n\n      @impl true\n      def storage(%User{}, _field) do\n        if test_environment?() do\n          {Uploadex.TestStorage, []}\n        else\n          {Uploadex.FileStorage, [...]}\n        end\n      end\n\n      @impl true\n      def accepted_extensions(%User{}, _field), do: ~w(.jpg .png)\n\n      defp test_environment?() do\n        # This env must be set in your config files depending on the environment.\n        Application.fetch_env!(:my_app, :environment) == :test\n      end\n    end","ref":"Uploadex.TestStorage.html#module-configuring"},{"type":"module","title":"Using - Uploadex.TestStorage","doc":"In your `ExUnit` tests, add a `setup` block starting the storage:\n\n    setup do\n      Uploadex.TestStorage.start_link()\n      :ok\n    end\n\nThen, in the tests, you can use the `get_stored/1`, `get_deleted/1` and `get_opts`:\n\n    test \"some test with files\" do\n      assert [\"document-1.pdf\", \"document-2.pdf\"] == Uploadex.TestStorage.get_stored()\n      assert [\"deleted-document.pdf\"] == Uploadex.TestStorage.get_deleted()\n      assert [] == Uploadex.TestStorage.get_opts()\n    end\n\nThis module is just the base for testing, check `Uploadex.Testing` for a more convenient way to test uploads.","ref":"Uploadex.TestStorage.html#module-using"},{"type":"function","title":"Uploadex.TestStorage.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Uploadex.TestStorage.html#child_spec/1"},{"type":"function","title":"Uploadex.TestStorage.get_deleted/1","doc":"","ref":"Uploadex.TestStorage.html#get_deleted/1"},{"type":"function","title":"Uploadex.TestStorage.get_opts/1","doc":"","ref":"Uploadex.TestStorage.html#get_opts/1"},{"type":"function","title":"Uploadex.TestStorage.get_stored/1","doc":"","ref":"Uploadex.TestStorage.html#get_stored/1"},{"type":"function","title":"Uploadex.TestStorage.start_link/1","doc":"","ref":"Uploadex.TestStorage.html#start_link/1"},{"type":"module","title":"Uploadex.Testing","doc":"This module simplifies testing and assertions involving `Uploadex`.","ref":"Uploadex.Testing.html"},{"type":"module","title":"Usage in Tests - Uploadex.Testing","doc":"The most convenient way to use `Uploadex.Testing` is to `use` the module:\n\n    use Uploadex.Testing\n\nThat will define all the helper functions you'll need to make assertions.","ref":"Uploadex.Testing.html#module-usage-in-tests"},{"type":"module","title":"Some examples of how to use the functions - Uploadex.Testing","doc":"```elixir\n# Asserting that 2 files were stored\nassert_stored_files_count(2)\n\n# Asserting the filenames of the stored files\nassert_stored_files([\"file1.pdf\", \"file2.pdf\"])\n\n# Asserting the filenames of the stored files ignoring the order\nassert_stored_files([\"file2.pdf\", \"file1.pdf\"], ignore_order: true)\n\n# Asserting that no file was stored\nrefute_stored_files()\n\n# Getting all stored files\n[\"file1.pdf\", \"file2.pdf\"] == get_stored_files()\n\n# Asserting that 1 files was deleted\nassert_deleted_files_count(1)\n\n# Asserting the filenames of the deleted files\nassert_deleted([\"file3.pdf\", \"file4.pdf\", \"file5.pdf\"])\n\n# Asserting the filenames of the deleted files ignoring the order\nassert_deleted_files([\"file4.pdf\", \"file5.pdf\", \"file3.pdf\"], ignore_order: true)\n\n# Asserting that no file was deleted\nrefute_deleted_files()\n\n# Getting all deleted files\n[\"file3.pdf\", \"file4.pdf\", \"file5.pdf\"] == get_deleted_files()\n```\n\nIf you need a more low level API, you can directly use the `Uploadex.TestStorage` functions.","ref":"Uploadex.Testing.html#module-some-examples-of-how-to-use-the-functions"},{"type":"macro","title":"Uploadex.Testing.__using__/1","doc":"Imports all functions from `Uploadex.Testing`\nand defines a setup callback to start the `Uploadex.TestStorage`.","ref":"Uploadex.Testing.html#__using__/1"},{"type":"function","title":"Uploadex.Testing.assert_deleted_files/2","doc":"Asserts that all the files in `expected_files` were deleted\nin the exact order that they are passed in the list.","ref":"Uploadex.Testing.html#assert_deleted_files/2"},{"type":"function","title":"Options - Uploadex.Testing.assert_deleted_files/2","doc":"* `ignoring_order`: if `true`, ignores the order of the files when asserting (defaults to `false`)","ref":"Uploadex.Testing.html#assert_deleted_files/2-options"},{"type":"function","title":"Uploadex.Testing.assert_deleted_files_count/1","doc":"Asserts that the given `expected_count` number of files was deleted.","ref":"Uploadex.Testing.html#assert_deleted_files_count/1"},{"type":"function","title":"Uploadex.Testing.assert_stored_files/2","doc":"Asserts that all the files in `expected_files` were stored\nin the exact order that they are passed in the list.","ref":"Uploadex.Testing.html#assert_stored_files/2"},{"type":"function","title":"Options - Uploadex.Testing.assert_stored_files/2","doc":"* `ignoring_order`: if `true`, ignores the order of the files when asserting (defaults to `false`)","ref":"Uploadex.Testing.html#assert_stored_files/2-options"},{"type":"function","title":"Uploadex.Testing.assert_stored_files_count/1","doc":"Asserts that the given `expected_count` number of files was stored.","ref":"Uploadex.Testing.html#assert_stored_files_count/1"},{"type":"function","title":"Uploadex.Testing.get_deleted_files/0","doc":"Returns all the deleted files.\n\nIt's the same as calling `Uploadex.TestStorage.get_deleted/0`.","ref":"Uploadex.Testing.html#get_deleted_files/0"},{"type":"function","title":"Uploadex.Testing.get_storage_opts/0","doc":"Returns the options used in the `Uploadex.TestStorage`.\n\nIt's the same as calling `Uploadex.TestStorage.get_opts/0`.","ref":"Uploadex.Testing.html#get_storage_opts/0"},{"type":"function","title":"Uploadex.Testing.get_stored_files/0","doc":"Returns all the stored files.\n\nIt's the same as calling `Uploadex.TestStorage.get_stored/0`.","ref":"Uploadex.Testing.html#get_stored_files/0"},{"type":"function","title":"Uploadex.Testing.refute_deleted_files/0","doc":"Asserts that no file was deleted.","ref":"Uploadex.Testing.html#refute_deleted_files/0"},{"type":"function","title":"Uploadex.Testing.refute_stored_files/0","doc":"Asserts that no file was stored.","ref":"Uploadex.Testing.html#refute_stored_files/0"},{"type":"function","title":"Uploadex.Testing.start_test_storage/1","doc":"Starts the `Uploadex.TestStorage`.\n\nAfter importing it, you can use as a `ExUnit` setup callback:\n\n    import Uploadex.Testing\n\n    setup :start_test_storage","ref":"Uploadex.Testing.html#start_test_storage/1"},{"type":"module","title":"Uploadex.Upload","doc":"Ecto type that handles upload.\n\nIt stores the filename in the database.","ref":"Uploadex.Upload.html"},{"type":"type","title":"Uploadex.Upload.upload_binary/0","doc":"","ref":"Uploadex.Upload.html#t:upload_binary/0"},{"type":"type","title":"Uploadex.Upload.upload_path/0","doc":"","ref":"Uploadex.Upload.html#t:upload_path/0"},{"type":"behaviour","title":"Uploadex.Uploader","doc":"Behaviour of an Uploader.","ref":"Uploadex.Uploader.html"},{"type":"behaviour","title":"Example - Uploadex.Uploader","doc":"defmodule MyApp.Uploader do\n      @moduledoc false\n      @behaviour Uploadex.Uploader\n\n      alias MyAppWeb.Endpoint\n\n      @impl true\n      def get_fields(%User{}), do: :photo\n      def get_fields(%Company{}), do: [:photo]\n\n      @impl true\n      def default_opts(Uploadex.FileStorage), do: [base_path: :code.priv_dir(:my_app), base_url: Endpoint.url()]\n      def default_opts(Uploadex.S3Storage), do: [bucket: \"my_bucket\", region: \"sa-east-1\", upload_opts: [acl: :public_read]]\n\n      @impl true\n      def storage(%User{id: id}, :photo), do: {Uploadex.FileStorage, directory: \"/uploads/users/#{id}\"}\n      def storage(%Company{}, :photo), do: {Uploadex.S3Storage, directory: \"/thumbnails\"}\n\n      # Optional:\n      @impl true\n      def accepted_extensions(%User{}, _field), do: ~w(.jpg .png)\n      def accepted_extensions(_any, _any), do: :any\n    end","ref":"Uploadex.Uploader.html#module-example"},{"type":"callback","title":"Uploadex.Uploader.accepted_extensions/2","doc":"","ref":"Uploadex.Uploader.html#c:accepted_extensions/2"},{"type":"callback","title":"Uploadex.Uploader.default_opts/1","doc":"","ref":"Uploadex.Uploader.html#c:default_opts/1"},{"type":"callback","title":"Uploadex.Uploader.get_fields/1","doc":"","ref":"Uploadex.Uploader.html#c:get_fields/1"},{"type":"callback","title":"Uploadex.Uploader.storage/2","doc":"","ref":"Uploadex.Uploader.html#c:storage/2"},{"type":"type","title":"Uploadex.Uploader.file/0","doc":"","ref":"Uploadex.Uploader.html#t:file/0"},{"type":"type","title":"Uploadex.Uploader.record/0","doc":"","ref":"Uploadex.Uploader.html#t:record/0"},{"type":"type","title":"Uploadex.Uploader.record_field/0","doc":"","ref":"Uploadex.Uploader.html#t:record_field/0"},{"type":"type","title":"Uploadex.Uploader.t/0","doc":"","ref":"Uploadex.Uploader.html#t:t/0"},{"type":"extras","title":"Uploadex","doc":"# Uploadex\n\nUploadex is an Elixir library for handling uploads that integrates well with [Ecto](https://github.com/elixir-ecto/ecto), [Phoenix](https://github.com/phoenixframework/phoenix) and [Absinthe](https://github.com/absinthe-graphql/absinthe).\n\nDocumentation can be found at https://hexdocs.pm/uploadex.","ref":"readme.html"},{"type":"extras","title":"Migrating from v2 to v3 - Uploadex","doc":"1. In you uploader, change `@behaviour Uploadex.Uploader` to `use Uploadex`\n1. Remove all `config :uploadex` from your configuration files\n1. Change all direct functions calls from `Uploadex.Resolver`, `Uploadex.Files` and `Uploadex` to your Uploader module","ref":"readme.html#migrating-from-v2-to-v3"},{"type":"extras","title":"Installation - Uploadex","doc":"The package can be installed by adding `uploadex` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:uploadex, \"~> 3.1.0\"},\n    # S3 dependencies(required for S3 storage only)\n    {:ex_aws, \"~> 2.1\"},\n    {:ex_aws_s3, \"~> 2.0.2\"},\n    {:sweet_xml, \"~> 0.6\"},\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Uploadex","doc":"Follow these steps to use Uploadex:\n\n#","ref":"readme.html#usage"},{"type":"extras","title":"1: Uploader - Uploadex","doc":"This library relies heavily on pattern matching for configuration, so the first step is to define your Uploader configuration module:\n\n```elixir\ndefmodule MyApp.Uploader do\n  @moduledoc false\n\n  use Uploadex,\n    repo: MyApp.Repo # only necessary if using the functions from Uploadex.Context\n\n  alias MyAppWeb.Endpoint\n\n  @impl true\n  def get_fields(%User{}), do: :photo\n  def get_fields(%Company{}), do: [:photo, :logo]\n\n  @impl true\n  def default_opts(Uploadex.FileStorage), do: [base_path: Path.join(:code.priv_dir(:my_app), \"static/\"), base_url: Endpoint.url()]\n  def default_opts(Uploadex.S3Storage), do: [bucket: \"my_bucket\", region: \"sa-east-1\", upload_opts: [acl: :public_read]]\n\n  @impl true\n  def storage(%User{id: id}, :photo), do: {Uploadex.FileStorage, directory: \"/uploads/users/#{id}\"}\n  def storage(%Company{id: id}, :photo), do: {Uploadex.S3Storage, directory: \"/thumbnails/#{id}\"}\n  def storage(%Company{}, :logo), do: {Uploadex.S3Storage, directory: \"/logos\"}\n\n  # Optional:\n  @impl true\n  def accepted_extensions(%User{}, :photo), do: ~w(.jpg .png)\n  def accepted_extensions(_any, _field), do: :any\nend\n```\n\nThis example shows the configuration for the [Uploadex.FileStorage](https://hexdocs.pm/uploadex/Uploadex.FileStorage.html#content) and [Uploadex.S3Storage](https://hexdocs.pm/uploadex/Uploadex.S3Storage.html#content) implementations, but you are free to implement your own [Storage](https://hexdocs.pm/uploadex/Uploadex.Storage.html#content).\n\n*Note: To avoid too much metaprogramming magic, the `use` in this module is very simple and, in fact, optional. If you wish to do so, you can just define the `@behaviour Uploadex.Uploader` instead of the `use` and then call all lower level modules directly, passing your Uploader module as argument. The `use` makes life much easier, though!*\n\n#","ref":"readme.html#1-uploader"},{"type":"extras","title":"2: Ecto Migration - Uploadex","doc":"A string field is required in the database to save the file reference.\nThe example below shows what would be needed to have a field to upload.\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddPhotoToUsers do\n  use Ecto.Migration\n\n  def change do\n    alter table(:users) do\n      add :photo, :string\n    end\n  end\nend\n```\n\n#","ref":"readme.html#2-ecto-migration"},{"type":"extras","title":"3: Schema - Uploadex","doc":"In your schema, use the Ecto Type [Uploadex.Upload](https://hexdocs.pm/uploadex/Uploadex.Upload.html#content):\n\n```elixir\nschema \"users\" do\n  field :name, :string\n  field :photo, Uploadex.Upload\nend\n\n# No special cast is needed, and casting does not have any side effects.\ndef create_changeset(%User{} = user, attrs) do\n  user\n  |> cast(attrs, [:name, :photo])\nend\n```\n\n#","ref":"readme.html#3-schema"},{"type":"extras","title":"4: Configuration - Uploadex","doc":"Depending on which features you are using, you may need extra configurations:\n\n##","ref":"readme.html#4-configuration"},{"type":"extras","title":"S3 Configuration - Uploadex","doc":"If you are using the S3 adapter, add this to your configuration file. For more information access the [ex_aws_s3 documentation](https://github.com/ex-aws/ex_aws_s3):\n\n```elixir\nconfig :ex_aws, :s3,\n  access_key_id: \"key\",\n  secret_access_key: \"secret\",\n  region: \"us-east-1\",\n  host: \"localhost\",\n  port: \"9000\",\n  scheme: \"http://\"\n\nconfig :my_project, :uploads,\n  bucket: \"uploads\",\n  region: \"us-east-1\"\n```\n\n#","ref":"readme.html#s3-configuration"},{"type":"extras","title":"5: Enjoy! - Uploadex","doc":"Now, you can use your defined Uploader to handle your records with their files!\n\nThe `use Uploadex` line in your Uploader module will import 3 groups of functions:\n\n##","ref":"readme.html#5-enjoy"},{"type":"extras","title":"Context - Uploadex","doc":"The highest level functions are context helpers (see [Context](https://hexdocs.pm/uploadex/Uploadex.Context.html) for more documentation), which will allow you to easily create, update and delete your records with associated files:\n\n  ```elixir\n  defmodule MyApp.Accounts do\n    alias MyApp.Accounts.User\n    alias MyApp.MyUploader\n\n    def create_user(attrs) do\n      %User{}\n      |> User.create_changeset(attrs)\n      |> MyUploader.create_with_file()\n    end\n\n    def update_user(%User{} = user, attrs) do\n      user\n      |> User.update_changeset(attrs)\n      |> MyUploader.update_with_file(user)\n    end\n\n    def delete_user(%User{} = user) do\n      MyUploader.delete_with_file(user)\n    end\n  end\n  ```\n\n##","ref":"readme.html#context"},{"type":"extras","title":"Resolver - Uploadex","doc":"There are also functions to help you easily fetch the files in Absinthe schemas:\n\n  ```elixir\n  object :user do\n    field :photo_url, :string, resolve: MyUploader.get_file_url(:photo)\n  end\n\n  object :user do\n    field :photos, list_of(:string), resolve: MyUploader.get_files_url(:photos)\n  end\n  ```\n\n  See [Resolver](https://hexdocs.pm/uploadex/Uploadex.Resolver.html#content) for more documentation.\n\n##","ref":"readme.html#resolver"},{"type":"extras","title":"Files - Uploadex","doc":"If you need more flexibility, you can use the lower-level functions defined in [Files](https://hexdocs.pm/uploadex/Uploadex.Files.html#content), which provide some extra functionalities, such as `get_temporary_file`, useful when the files are not publicly available.\n\nSome examples:\n\n```elixir\n{:ok, %User{}} = MyUploader.store_files(user)\n{:ok, %User{}} = MyUploader.delete_files(user)\n{:ok, %User{}} = MyUploader.delete_previous_files(user, user_after_change)\n{:ok, files} = MyUploader.get_files_url(user, :photos)\n```","ref":"readme.html#files"},{"type":"extras","title":"Testing - Uploadex","doc":"For knowing how to test with Uploadex, check the hexdocs of the [Testing](https://hexdocs.pm/uploadex/Uploadex.Testing.html#content) module.","ref":"readme.html#testing"},{"type":"extras","title":"Motivation - Uploadex","doc":"Even though there already exists a library for uploading files that integrates with ecto (https://github.com/stavro/arc_ecto), this library was created because:\n\n* arc_ecto does not support upload of binary files\n* Uploadex makes it easier to deal with records that contain files without having to manage those files manually on every operation\n* Using uploadex, the changeset operations have no side-effects and no special casting is needed\n* Uploadex offers more flexibility by allowing to define different storage configurations for each struct (or even each field in a struct) in the application\n* Uploadex does not rely on global configuration, which makes it easier to work in umbrella applications","ref":"readme.html#motivation"}],"content_type":"text/markdown"}